from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageEvent

from ..database.models import Server
from ..utils import permission_check, to_me

servers = on_command("服务器列表", rule=to_me(), priority=3)

servers.handle()(permission_check)


@servers.handle()
async def _(event: MessageEvent):
    """发送数据库中所有服务器"""
    message = "数据库中的服务器列表\n\n"
    server_list = Server.all()
    async for server in server_list:
        message += (
            f"{server.server_name}："
            f"Rcon_Msg：{'开' if server.rcon_msg else '关'}，"
            f"Rcon_CMD：{'开' if server.rcon_cmd else '关'}，\n"
        )
    await servers.finish(message=message)



import aiomcrcon
from nonebot import logger

from . import change_ip, change_port, change_password
from ..on_msg.data_source import CLIENTS, rcon_connect
from ...database import DB as db


async def change_rcon_status(server_name):
    """更新Rcon状态"""
    for client in CLIENTS:
        # 如果服务器名匹配
        if client["server_name"] == server_name:
            # 获取该服务器的信息
            server = await db.get_server(server_name=server_name)
            # 开
            if server.rcon_msg or server.rcon_cmd and not client["rcon_connection"]["is_open"]:
                client["rcon_connection"]["rcon"] = aiomcrcon.Client(
                    server.rcon_ip,
                    server.rcon_port,
                    server.rcon_password
                )
                await rcon_connect(client=client["rcon_connection"]["rcon"], server_name=server_name)

            # 关：rcon消息关、rcon命令关、rcon状态开
            if not server.rcon_msg and not server.rcon_cmd and client["rcon_connection"]["is_open"]:
                await client["rcon_connection"]["rcon"].close()
                client["rcon_connection"]["is_open"] = False
                logger.success(f"[MC_QQ]丨[Server:{server.server_name}] 的Rcon已关闭")



from typing import Union

from nonebot import on_command
from nonebot.adapters.onebot.v11 import GroupMessageEvent
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database import DB as db

from ...utils import (
    get_type_id,
    permission_check,
    to_me
)

sub_list = on_command("互通列表", rule=to_me(), priority=3)
sub_list.__doc__ = """互通列表（群聊丨管理员）"""

sub_list.handle()(permission_check)


@sub_list.handle()
async def _(event: Union[GroupMessageEvent, GuildMessageEvent]):
    """发送当前位置的互通列表"""
    message = "互通列表（所有群/频道都是分开的）\n\n"
    subs = await db.get_sub_list(event.message_type, await get_type_id(event))

    for sub in subs:
        server = await db.get_server(server_name=sub.server_name)
        assert server is not None
        message += (
            f"{sub.server_name} "
            f"显示服务器名：{'开' if sub.display_server_name else '关'}，"
        )
    await sub_list.finish(message)



from typing import Union

from nonebot import on_command
from nonebot.adapters.onebot.v11.event import GroupMessageEvent
from nonebot.params import ArgPlainText
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database import DB as db
from ...utils import get_type_id, permission_check, handle_server_name, server_name_check, to_me

display_on = on_command("开启服务器名", rule=to_me(), priority=3)
display_on.__doc__ = """开启服务器名 Server（管理员）"""

display_on.handle()(permission_check)

display_on.handle()(handle_server_name)

display_on.got("server_name", prompt="请输入要开启显示服务器名的服务器")(server_name_check)


@display_on.handle()
async def _(event: Union[GroupMessageEvent, GuildMessageEvent], server_name: str = ArgPlainText("server_name")):
    """根据 服务器名 开启 显示服务器名"""
    if await db.set_sub(
            "display_server_name",
            True,
            server_name=server_name,
            type=event.message_type,
            type_id=await get_type_id(event)
    ):
        server = await db.get_server(server_name=server_name)
        assert server is not None
        await display_on.finish(f"已开启 [{server.server_name}] 的显示服务器名")
    await display_on.finish(f"服务器（{server_name}）未互通，请先互通后再操作")



from nonebot import get_driver


def get_mc_qq_ws_ip() -> str:
    """获取 WebSocket IP"""
    try:
        return str(get_driver().config.mc_qq_ws_ip)
    except AttributeError:
        return "localhost"


def get_mc_qq_ws_port() -> int:
    """获取 WebSocket 端口"""
    try:
        return int(get_driver().config.mc_qq_ws_port)
    except AttributeError:
        return 8765


def get_mc_qq_send_group_name() -> bool:
    """获取 是否发送群聊名称"""
    try:
        return bool(get_driver().config.mc_qq_send_group_name)
    except AttributeError:
        return False


def get_mc_qq_display_server_name() -> bool:
    """获取 是否显示服务器名称"""
    try:
        return bool(get_driver().config.mc_qq_display_server_name)
    except AttributeError:
        return False


def get_mc_qq_servers_list() -> list:
    """获取 服务器列表"""
    try:
        return list(get_driver().config.mc_qq_servers_list)
    except AttributeError:
        return []


def get_mc_qq_mcrcon_guild_admin_roles() -> list:
    """获取频道 MC_QQ 管理身份组"""
    try:
        return list(get_driver().config.mc_qq_mcrcon_guild_admin_roles)
    except AttributeError:
        return ["频道主", "管理员"]


def get_mc_qq_dir() -> str:
    """获取 MC_QQ 数据库路径"""
    try:
        return str(get_driver().config.mc_qq_dir)
    except AttributeError:
        return ""


def get_mc_qq_to_me() -> bool:
    """获取 MC_QQ 数据库路径"""
    try:
        return bool(get_driver().config.mc_qq_to_me)
    except AttributeError:
        return True



import websockets
from nonebot import logger

from .plugins.on_msg.data_source import ws_client
from .utils import config


async def start_ws_server():
    """启动 WebSocket 服务器"""
    global ws
    ws = await websockets.serve(ws_client, config.get_mc_qq_ws_ip(), config.get_mc_qq_ws_port())
    logger.success("[MC_QQ]丨WebSocket 服务器已开启")


async def stop_ws_server():
    """关闭 WebSocket 服务器"""
    global ws
    ws.close()
    logger.success("[MC_QQ]丨WebSocket 服务器已关闭")



from pathlib import Path

import click
import dotenv

env = {
    "HOST": "127.0.0.1",
    "PORT": "8080",
    "SECRET": "",
    "ACCESS_TOKEN": "",
    "SUPERUSERS": [],
    "COMMAND_START": [""],
    "MC_QQ_DIR": "./data/",
}


def create_env():
    file_path = Path("./.env.prod").resolve()
    if file_path.exists():
        return

    if any(Path.cwd().iterdir()):
        print("文件夹不为空，请更换空文件夹后重试")
        import sys
        sys.exit()

    while True:
        try:
            superusers = click.prompt(
                "主人QQ号或频道ID (多个使用空格分开)", default="", show_default=False
            )
            superusers = [int(superuser) for superuser in superusers.split()]
            env["SUPERUSERS"] = superusers
            break
        except ValueError:
            print("输入格式有误，请重新输入")

    with open(".env.prod", "w"):
        pass

    for key, value in env.items():
        dotenv.set_key(
            file_path,
            key,
            str(value).replace("'", '"').replace(" ", ""),
            quote_mode="never",
        )


if __name__ == "__main__":
    create_env()



from typing import Union

from nonebot import on_command
from nonebot.adapters.onebot.v11.event import GroupMessageEvent
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database import DB as db
from ...utils import permission_check, to_me

send_off = on_command("关闭发送群名", rule=to_me(), priority=3)
send_off.__doc__ = """关闭发送群名（群聊丨管理员）"""

send_off.handle()(permission_check)


@send_off.handle()
async def _(event: Union[GroupMessageEvent, GuildMessageEvent]):
    """关闭 发送群名"""
    if isinstance(event, GroupMessageEvent):
        if await db.update_group(group_id=event.group_id, switch=False):
            group = await db.get_group(group_id=event.group_id)
            assert group is not None
    else:
        if await db.update_guild(guild_id=event.guild_id, channel_id=event.channel_id, switch=False):
            guild = await db.get_guild(guild_id=event.guild_id, channel_id=event.channel_id)
            assert guild is not None
    await send_off.finish(f"已关闭本群聊的发送群名")



import sys
from os import path

import nonebot
from nonebot.adapters.onebot.v11 import Adapter
from nonebot.log import default_format, logger

nonebot.init()
driver = nonebot.get_driver()
driver.register_adapter(Adapter)
app = nonebot.get_asgi()

# 删除 qinglan_bot 导入，否则 nonebot 导入时会忽略
del sys.modules["qinglan_bot"]
nonebot.load_plugin("qinglan_bot")
nonebot.load_plugin("nonebot_plugin_gocqhttp")

# Modify some config / config depends on loaded configs
#
# config = nonebot.get_driver().config
# do something...

logger.add(
    path.join("log", "error.log"),
    rotation="00:00",
    retention="1 week",
    diagnose=False,
    level="ERROR",
    format=default_format,
    encoding="utf-8",
)


def run():
    nonebot.run(app="qinglan_bot.cli.bot:app")



import json
from typing import Union

from nonebot import logger
from nonebot.exception import FinishedException
from nonebot.internal.permission import Permission
from nonebot.internal.rule import Rule
from nonebot.matcher import Matcher
from nonebot.params import ArgPlainText, CommandArg
from nonebot.adapters.onebot.v11 import Message, GroupMessageEvent, Bot, GROUP_ADMIN, GROUP_OWNER, PrivateMessageEvent
from nonebot.permission import SUPERUSER
from nonebot_plugin_guild_patch import ChannelDestroyedNoticeEvent, GuildMessageEvent

from .config import get_mc_qq_mcrcon_guild_admin_roles, get_mc_qq_to_me
from ..database import DB as db
from ..database.db import server_list


def to_me():
    if get_mc_qq_to_me():
        from nonebot.rule import to_me

        return to_me()

    async def _to_me() -> bool:
        return True

    return Rule(_to_me)


async def _guild_admin(bot: Bot, event: GuildMessageEvent):
    roles = set(
        role["role_name"]
        for role in (
            await bot.get_guild_member_profile(
                guild_id=event.guild_id, user_id=event.user_id
            )
        )["roles"]
    )
    return bool(roles & set(get_mc_qq_mcrcon_guild_admin_roles()))


GUILD_ADMIN: Permission = Permission(_guild_admin)


async def permission_check(
        bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent, PrivateMessageEvent]
):
    # from ..database import DB as db

    if isinstance(event, PrivateMessageEvent):
        if event.sub_type == "group":  # 不处理群临时会话
            raise FinishedException
        if await SUPERUSER(bot, event):
            return

    if isinstance(event, GroupMessageEvent):
        # if not await db.get_group_admin(event.group_id):
        #     return
        if await (GROUP_ADMIN | GROUP_OWNER | SUPERUSER)(bot, event):
            return
    elif isinstance(event, GuildMessageEvent):
        # if not await db.get_guild_admin(event.guild_id, event.channel_id):
        #     return
        if await (GUILD_ADMIN | SUPERUSER)(bot, event):
            return
    await bot.send(event, "权限不足，目前只有管理员才能使用")
    raise FinishedException


async def handle_server_name(
        matcher: Matcher,
        command_arg: Message = CommandArg(),
):
    server_name = command_arg.extract_plain_text().strip()
    if server_name:
        matcher.set_arg("server_name", command_arg)


async def server_name_check(
        matcher: Matcher,
        server_name: str = ArgPlainText("server_name"),
):
    server_name = server_name.strip()
    matcher.set_arg("server_name", Message(server_name))


async def get_type_id(event: Union[GroupMessageEvent, GuildMessageEvent, ChannelDestroyedNoticeEvent]):
    if isinstance(event, GuildMessageEvent) or isinstance(event, ChannelDestroyedNoticeEvent):
        from ..database import DB as db
        return await db.get_guild_type_id(event.guild_id, event.channel_id)
    return event.group_id


async def msg_rule(event: Union[GroupMessageEvent, GuildMessageEvent]) -> bool:
    """Rule 消息规则"""
    if server_list:
        for per_server in server_list:
            for per_group in per_server["all_group_list"]:
                if await get_type_id(event) == per_group["type_id"]:
                    return True
    return False


async def msg_to_qq_process(json_msg):
    """处理来自MC的消息，并返回处理后的消息"""
    message = {
        "PlayerJoinEvent": f"{json_msg['player']['nickname']} 加入了服务器",
        "PlayerQuitEvent": f"{json_msg['player']['nickname']} 离开了服务器",
        "AsyncPlayerChatEvent": f"{json_msg['player']['nickname']} 说：{json_msg['message']}",
        "PlayerDeathEvent": f"{json_msg['message']}",
    }
    return message[json_msg['event_name']]


async def send_msg_to_qq(bot: Bot, json_msg):
    """发送消息到 QQ"""
    msg = await msg_to_qq_process(json_msg)
    # 循环服务器列表并发送消息
    # 如果服务器列表不为空
    if server_list:
        # 开始循环
        for per_server in server_list:
            # 如果服务器名相匹配
            if per_server['server_name'] == json_msg['server_name']:
                # 如果全群聊列表存在
                if per_server['all_group_list']:
                    # 循环全群聊列表
                    for per_group in per_server['all_group_list']:
                        # 是否需要发送服务器名称
                        if per_group["display_server_name"]:
                            msg = f"[{json_msg['server_name']}] {msg}"
                        # 发送至群聊
                        if per_group["type"] == "group":
                            logger.success(
                                f"[MC_QQ]丨from [{json_msg['server_name']}] to [群:{per_group['type_id']}] \"{msg}\"")
                            await bot.send_group_msg(
                                group_id=per_group['type_id'],
                                message=msg
                            )
                        # 发送至频道
                        else:
                            guild = await db.get_guild(id=per_group['type_id'])
                            logger.success(
                                f"[MC_QQ]丨from [{json_msg['server_name']}] to [频道:{guild.guild_id}/{guild.channel_id}] \"{msg}\"")
                            await bot.send_guild_channel_msg(
                                guild_id=guild.guild_id,
                                channel_id=guild.channel_id,
                                message=msg
                            )


async def get_member_nickname(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent], user_id):
    """获取昵称"""
    # 判断从 群/频道 获取成员信息
    if isinstance(event, GroupMessageEvent):
        # 如果获取发送者的昵称
        if event.user_id == int(user_id):
            return event.sender.card if event.sender.card else event.sender.nickname
        # 如果获取其他人的昵称
        else:
            return (await bot.get_group_member_info(
                group_id=event.group_id,
                user_id=user_id,
                no_cache=True
            ))['nickname']
    elif isinstance(event, GuildMessageEvent):
        # 返回频道成员昵称
        if event.user_id == user_id:
            return event.sender.nickname
        else:
            return (await bot.get_guild_member_profile(
                guild_id=event.guild_id,
                user_id=user_id))['nickname']


async def process_msg_for_ws(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """处理发送至MC的信息"""
    # 获取昵称
    member_nickname = await get_member_nickname(bot, event, event.user_id)

    # 初始化消息
    text_msg = member_nickname + "说："

    # 初始化消息字典
    messageList = []

    # 发送群聊名称
    from ..database import DB as db
    if isinstance(event, GroupMessageEvent) and (await db.get_group(group_id=event.group_id)).send_group_name:
        group_name = {'msgType': "group_name",
                      'msgData': (await bot.get_group_info(group_id=event.group_id))['group_name']}
        messageList.append(group_name)
    elif isinstance(event, GuildMessageEvent) and (
            await db.get_guild(guild_id=event.guild_id, channel_id=event.channel_id)).send_group_name:
        group_name = {'msgType': "group_name"}
        guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
        for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
            if str(event.channel_id) == per_channel['channel_id']:
                channel_name = per_channel['channel_name']
                group_name['msgData'] = f"{guild_name}丨{channel_name}"
                break
        messageList.append(group_name)

    # 将群成员昵称装入消息列表
    messageList.append({"msgType": "senderName", "msgData": member_nickname})

    for msg in event.message:
        per_msg = {'msgType': msg.type}
        # 文本
        if msg.type == "text":
            msgData = msg.data['text'].replace("\r", "").replace("\n", "\n * ")
            text_msg += msgData
        # 图片
        elif msg.type == "image":
            msgData = msg.data['url']
            text_msg += '[图片]'
        # 表情
        elif msg.type == "face":
            msgData = '[表情]'
            text_msg += '[表情]'
        # 语音
        elif msg.type == "record":
            msgData = '[语音]'
            text_msg += '[语音]'
        # 视频
        elif msg.type == "video":
            msgData = msg.data['url']
            text_msg += '[视频]'
        # @
        elif msg.type == "at":
            # 获取被@ 群/频道 昵称
            at_member_nickname = await get_member_nickname(bot, event, msg.data['qq'])
            msgData = f"@{at_member_nickname}"
            text_msg += msgData
        # share
        elif msg.type == "share":
            msgData = msg.data['url']
            text_msg += '[分享：' + msg.data['title'] + ']'
        # forward
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            msgData = '[合并转发]'
            text_msg = msgData
        else:
            msgData = msg.type
            text_msg += '[' + msg.type + '] '

        text_msg += " "

        # 装入消息数据
        per_msg['msgData'] = msgData
        # 放入消息列表
        messageList.append(per_msg)

    return text_msg, '{"message": ' + str(messageList) + '}'


async def process_msg_for_rcon(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """消息处理"""
    # 获取昵称
    member_nickname = await get_member_nickname(bot, event, event.user_id)

    # 初始化日志消息
    text_msg = member_nickname + " 说："

    command_msg = "tellraw @p "

    message_list = [
        {"text": "[MC_QQ] ", "color": "yellow"},
    ]
    # 是否发送群聊名称
    from ..database import DB as db
    # 群
    if isinstance(event, GroupMessageEvent) and (await db.get_group(group_id=event.group_id)).send_group_name:
        message_list.append(
            {"text": (await bot.get_group_info(group_id=event.group_id))['group_name'] + " ", "color": "aqua"})
    # 频道
    elif isinstance(event, GuildMessageEvent) and (
            await db.get_guild(guild_id=event.guild_id, channel_id=event.channel_id)).send_group_name:
        guild_name = (await bot.get_guild_meta_by_guest(guild_id=event.guild_id))['guild_name']
        for per_channel in (await bot.get_guild_channel_list(guild_id=event.guild_id, no_cache=True)):
            if str(event.channel_id) == per_channel['channel_id']:
                message_list.append({"text": guild_name + "丨" + per_channel['channel_name'] + " ", "color": "aqua"})
                break
    message_list.append({"text": member_nickname, "color": "aqua"})
    message_list.append({"text": " 说：", "color": "yellow"})

    for msg in event.message:
        # 文本
        if msg.type == "text":
            msg_dict = {"text": msg.data['text'].replace("\r", "").replace("\n", "\n * ") + " ", "color": "white"}
            text_msg += msg.data['text'].replace("\r", "").replace("\n", "\n * ")
        # 图片
        elif msg.type == "image":
            msg_dict = {"text": "[图片] ", "color": "yellow",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看图片", "color": "gold"}]}
                        }
            text_msg += '[图片]'
        # 表情
        elif msg.type == "face":
            msg_dict = {"text": "[表情] ", "color": "gold"}
            text_msg += '[表情]'
        # 语音
        elif msg.type == "record":
            msg_dict = {"text": "[语音] ", "color": "light_purple"}
            text_msg += '[语音]'
        # 视频
        elif msg.type == "video":
            msg_dict = {"text": "[视频] ", "color": "light_purple",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看视频", "color": "dark_purple"}]}
                        }
            text_msg += '[视频]'
        # @
        elif msg.type == "at":
            # 获取被@ 群/频道 昵称
            at_member_nickname = await get_member_nickname(bot, event, msg.data['qq'])
            msg_dict = {"text": "@" + at_member_nickname + " ", "color": "green"}
            text_msg += f"@{at_member_nickname}"
        # share
        elif msg.type == "share":
            msg_dict = {"text": "[分享：" + msg.data['title'] + "] ", "color": "yellow",
                        "clickEvent": {"action": "open_url", "value": msg.data['url']},
                        "hoverEvent": {"action": "show_text", "contents": [{"text": "查看图片", "color": "gold"}]}
                        }
            text_msg += '[分享：' + msg.data['title'] + ']'
        # forward
        elif msg.type == "forward":
            # TODO 将合并转发消息拼接为字符串
            # 获取合并转发 await bot.get_forward_msg(message_id=event.message_id)
            msg_dict = {"text": "[合并转发] ", "color": "white"}
            text_msg += '[合并转发]'
        else:
            msg_dict = {"text": "[ " + msg.type + "] ", "color": "white"}
            text_msg += '[' + msg.type + ']'

        # 放入消息列表
        message_list.append(msg_dict)

    # 拼接完整命令
    command_msg += json.dumps(message_list)
    return text_msg, command_msg



import json
import aiomcrcon
import websockets
from nonebot import get_bot, logger
from nonebot.adapters.onebot.v11 import Bot, GroupMessageEvent
from nonebot_plugin_guild_patch import GuildMessageEvent

from typing import Union, Optional
from ...utils import process_msg_for_ws, send_msg_to_qq, get_type_id, process_msg_for_rcon
from ...database.db import server_list

CLIENTS = []
"""客户端列表"""


async def ws_client(websocket):
    """WebSocket"""
    try:
        server_name = websocket.request_headers["x-self-name"].encode('utf-8').decode('unicode_escape')
    except KeyError:
        server_name = ""
    # 服务器名为空
    if not server_name:
        logger.error("[MC_QQ]丨未获取到该服务器的名称，连接断开")
        await websocket.close(1008, "[MC_QQ]丨未获取到该服务器的名称，连接断开")
        return
    else:
        for client in CLIENTS:
            # 重复连接
            if server_name == client["server_name"]:
                logger.error(f"[MC_QQ]丨[Server:{server_name}] 已连接至 WebSocket 服务器，无需重复连接")
                await websocket.close(1008, f"[MC_QQ]丨[Server:{server_name}] 已连接至 WebSocket 服务器，无需重复连接")
                return
    # 准备加入客户端列表
    try:
        from ...database import DB as db
        server = await db.get_server(server_name=server_name)
        # rcon状态：rcon_msg rcon_cmd rcon_password不等于默认密码
        rcon_status = (server.rcon_msg or server.rcon_cmd) and server.rcon_password != "change_password"
    except AttributeError:
        rcon_status = False

    if rcon_status:
        rcon_client = aiomcrcon.Client(server.rcon_ip, server.rcon_port, server.rcon_password)
        await rcon_connect(client=rcon_client, server_name=server_name)
    else:
        rcon_client: Optional[aiomcrcon.Client] = None

    CLIENTS.append({
        "server_name": server_name, "ws_client": websocket, "rcon_connection": {
            "rcon": rcon_client, "is_open": rcon_status
        }
    })
    logger.success(f"[MC_QQ]丨[Server:{server_name}] 已连接至 WebSocket 服务器")
    try:
        async for message in websocket:
            await send_msg_to_qq(bot=get_bot(), json_msg=json.loads(message))
    except websockets.WebSocketException:
        # 移除当前客户端
        await delete_client(server_name)
    if websocket.closed and CLIENTS:
        await delete_client(server_name)


async def send_msg_to_mc(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """发送消息到 MC"""
    # 处理来自QQ的消息
    if client_list := await get_clients(event=event):
        for client in client_list:
            # 如果 服务器的rcon已连接 且 服务器切换为rcon发送消息
            if client["client"]["rcon_connection"]["is_open"] and client["server"]["rcon_msg"] and \
                    client["client"]["rcon_connection"]["rcon"]:
                text_msg, msgJson = await process_msg_for_rcon(bot=bot, event=event)
                try:
                    await client["client"]["rcon_connection"]["rcon"].send_cmd(msgJson)
                    logger.success(f"[MC_QQ_Rcon]丨发送至 [server:{client['client']['server_name']}] 的消息 \"{text_msg}\"")
                except aiomcrcon.ClientNotConnectedError:
                    logger.error(f"[MC_QQ]丨[Server:{client['client']['server_name']}] 的Rcon未连接，发送消息失败")

            elif client["client"]['ws_client']:
                text_msg, msgJson = await process_msg_for_ws(bot=bot, event=event)
                try:
                    await client["client"]['ws_client'].send(msgJson)
                    logger.success(f"[MC_QQ]丨发送至 [server:{client['client']['server_name']}] 的消息 \"{text_msg}\"")
                except websockets.WebSocketException:
                    logger.error(f"[MC_QQ]丨发送至 [Server:{client['client']['server_name']}] 的过程中出现了错误")
                    CLIENTS.remove(client)


async def send_command_to_mc(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    """发送命令到 Minecraft"""
    if client_list := await get_clients(event=event):
        for client in client_list:
            if client["client"]["rcon_connection"]["is_open"] and client["server"]["rcon_cmd"] and client["client"]["rcon_connection"]["rcon"]:
                try:
                    await bot.send(event, message=str(
                        (await client["client"]["rcon_connection"]["rcon"].send_cmd(event.raw_message.strip("/mcc").strip()))[
                            0]))
                    logger.success(
                        f"[MC_QQ_Rcon]丨发送至 [server:{client['client']['server_name']}] 的命令 \"{event.raw_message.strip('/mcc').strip()}\""
                    )
                except aiomcrcon.ClientNotConnectedError:
                    logger.error(f"[MC_QQ_Rcon]丨发送至 [Server:{client['client']['server_name']}] 的过程中出现了错误")
                    # 连接关闭则移除客户端
                    CLIENTS.remove(client)


async def get_clients(event: Union[GroupMessageEvent, GuildMessageEvent]):
    """获取 服务器名、ws客户端、rcon连接"""
    res = []
    for per_client in CLIENTS:
        for per_server in server_list:
            # 如果 服务器名 == ws客户端中记录的服务器名，且ws客户端存在
            if per_client['ws_client'] and per_server['server_name'] == per_client['server_name']:
                for per_group in per_server['all_group_list']:
                    if await get_type_id(event) == per_group["type_id"]:
                        res.append({"client": per_client, "server": per_server})
    return res


async def connect_rcon():
    """服务器启动时，连接启用rcon的服务器"""
    for server in server_list:
        if (server.rcon_msg | server.rcon_cmd) and server.rcon_password != "change_password":
            rcon_client = aiomcrcon.Client(server.rcon_ip, server.rcon_port, server.rcon_password)
            try:
                await rcon_client.connect()
                # 连接成功后装入rcon连接列表
                for client in CLIENTS:
                    if server.server_name == client["server_name"]:
                        client["rcon_connection"]["rcon"] = rcon_client
                        client["rcon_connection"]["is_open"] = True
                    logger.success(f"[MC_QQ]丨[Server:{client['server_name']}] 的Rcon连接成功")
            except aiomcrcon.RCONConnectionError:
                logger.error(f"[MC_QQ]丨[Server:{server.server_name}] 的Rcon连接失败")
            except aiomcrcon.IncorrectPasswordError:
                logger.error(f"[MC_QQ]丨[Server:{server.server_name}] 的Rcon密码错误")


async def rcon_connect(client: aiomcrcon.Client, server_name: str):
    try:
        await client.connect()
        logger.success(f"[MC_QQ]丨[Server:{server_name}] 的Rcon连接成功")
    except aiomcrcon.RCONConnectionError:
        logger.error(f"[MC_QQ]丨[Server:{server_name}] 的Rcon连接失败")
    except aiomcrcon.IncorrectPasswordError:
        logger.error(f"[MC_QQ]丨[Server:{server_name}] 的Rcon密码错误")


async def delete_client(server_name: str):
    for client in CLIENTS:
        if server_name == client["server_name"]:
            if client["rcon_connection"]["is_open"]:
                await client["rcon_connection"]["rcon"].close()
                logger.error(f"[MC_QQ]丨[Server:{server_name}] 的 Rcon 连接已断开")
            CLIENTS.remove(client)
            logger.error(f"[MC_QQ]丨[Server:{server_name}] 的 WebSocket 连接已断开")



from nonebot import get_driver
from nonebot.plugin.manager import PluginLoader

if isinstance(globals()["__loader__"], PluginLoader):
    driver = get_driver()

    from .ws_server import start_ws_server, stop_ws_server
    from . import plugins


    # Bot 连接时
    @driver.on_bot_connect
    async def on_start():
        # 启动 WebSocket 服务器
        await start_ws_server()


    # Bot 断开时
    @driver.on_bot_disconnect
    async def on_close():
        # 关闭 WebSocket 服务器
        await stop_ws_server()



from nonebot import on_message, on_command
from nonebot.adapters.onebot.v11 import Bot, GroupMessageEvent
from nonebot_plugin_guild_patch import GuildMessageEvent

from .data_source import send_msg_to_mc, send_command_to_mc
from ...utils import msg_rule, permission_check
from typing import Union

mc_qq = on_message(priority=5, rule=msg_rule, block=False)
mc_qq_command = on_command(
    "mcc",
    priority=3,
    rule=msg_rule
)
mc_qq_command.handle()(permission_check)


# 收到 群/频 道消息时
@mc_qq.handle()
async def msg_first_receive(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    await send_msg_to_mc(bot=bot, event=event)


# 收到 群/频道 命令时
@mc_qq_command.handle()
async def cmd_first_receive(bot: Bot, event: Union[GroupMessageEvent, GuildMessageEvent]):
    await send_command_to_mc(bot=bot, event=event)



from nonebot import on_command
from nonebot.matcher import matchers

from ..utils import to_me

help = on_command("ql帮助", rule=to_me(), priority=3)


@help.handle()
async def _():
    message = "青岚 目前支持的功能：\n（请将Server替换为需要操作的服务器名）\n"
    for matchers_list in matchers.values():
        for matcher in matchers_list:
            if (
                    matcher.plugin_name
                    and matcher.plugin_name.startswith("qinglan_bot")
                    and matcher.__doc__
            ):
                message += matcher.__doc__ + "\n"
    message += "详细帮助：https://17theword.github.io/qinglan_bot"
    await help.finish(message)



from nonebot import on_command
from nonebot.params import ArgPlainText
from nonebot.permission import SUPERUSER
from nonebot.adapters.onebot.v11 import PrivateMessageEvent
from ...database import DB as db
from ...utils import handle_server_name, server_name_check, to_me

change_ip = on_command("修改rconip", rule=to_me(), permission=SUPERUSER, priority=3)
change_ip.__doc__ = """修改rconip Server IP（私聊丨超级用户）"""

change_ip.handle()(handle_server_name)

change_ip.got("server_name", prompt="请输入内容：服务器名 IP")(server_name_check)


@change_ip.handle()
async def _(event: PrivateMessageEvent, server_name: str = ArgPlainText("server_name")):
    """根据服务器名 修改 Rcon_IP"""
    if event.sub_type != "group":  # 不处理群临时会话
        arg_list = server_name.split()
        if await db.update_server(
                "rcon_ip",
                arg_list[1],
                server_name=arg_list[0],
        ):
            server = await db.get_server(server_name=arg_list[0])
            assert server is not None
            await change_ip.finish(f"修改 [{server.server_name}] 的Rcon_IP成功")
        await change_ip.finish(f"服务器（{arg_list[0]}）未互通，请先互通后再操作")



from typing import Union

from nonebot import on_command
from nonebot.adapters.onebot.v11 import GroupMessageEvent
from nonebot.internal.params import ArgPlainText
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database import DB as db

from ...utils import (
    handle_server_name,
    server_name_check,
    get_type_id,
    permission_check,
    to_me
)

add_sub = on_command("开启互通", rule=to_me(), priority=3)
add_sub.__doc__ = """开启互通 Server（群聊丨管理员）"""

add_sub.handle()(permission_check)

add_sub.handle()(handle_server_name)

add_sub.got("server_name", prompt="请输入要开启互通的服务器名")(server_name_check)


@add_sub.handle()
async def _(event: Union[GroupMessageEvent, GuildMessageEvent], server_name: str = ArgPlainText("server_name")):
    """根据服务器名 开启互通"""

    if isinstance(event, GuildMessageEvent):
        await db.add_guild(
            guild_id=event.guild_id, channel_id=event.channel_id, send_group_name=False
        )

    result = await db.add_sub(
        type=event.message_type,
        type_id=await get_type_id(event),
        server_name=server_name,
        display_server_name=False
    )

    if result:
        await add_sub.finish(message=f"已开启与服务器 {server_name} 的互通")
    await add_sub.finish(message=f"服务器 {server_name} 的互通已经开启过了")



from typing import Union

from nonebot import on_command
from nonebot.adapters.onebot.v11 import GroupMessageEvent
from nonebot.internal.params import ArgPlainText
from nonebot_plugin_guild_patch import GuildMessageEvent

from ...database import DB as db

from ...utils import (
    handle_server_name,
    server_name_check,
    get_type_id,
    permission_check,
    to_me
)

delete_sub = on_command("关闭互通", rule=to_me(), priority=3)
delete_sub.__doc__ = """关闭互通 Server（群聊丨管理员）"""

delete_sub.handle()(permission_check)

delete_sub.handle()(handle_server_name)

delete_sub.got("server_name", prompt="请输入要关闭互通的服务器名")(server_name_check)


@delete_sub.handle()
async def _(event: Union[GroupMessageEvent, GuildMessageEvent], server_name: str = ArgPlainText("server_name")):
    """根据服务器名 关闭互通"""
    servername = getattr(await db.get_server(server_name=server_name), "server_name", None)

    if servername:
        result = await db.delete_sub(
            type=event.message_type,
            type_id=await get_type_id(event),
            server_name=server_name
        )
    else:
        result = False

    if result:
        await delete_sub.finish(message=f"已关闭与服务器 {server_name} 的互通")
    await delete_sub.finish(message=f"服务器 {server_name} 的互通未开启")



from pathlib import Path
from typing import Optional, List

from nonebot import get_driver
from tortoise import Tortoise, connections
from .models import Sub, Server, Group, Guild
from ..utils.config import get_mc_qq_dir

server_list = []


def get_path(*other):
    """获取数据文件绝对路径"""

    dir_path = Path(get_mc_qq_dir() if get_mc_qq_dir() else "./data/")
    if not Path(dir_path).exists():
        dir_path.mkdir()
    dir_path = dir_path.resolve()
    return str(dir_path.joinpath(*other))


class DB:
    """数据库交互类，与增删改查无关的部分不应该在这里面实现"""

    @classmethod
    async def init(cls):
        """初始化数据库"""
        from . import models  # noqa: F401

        await Tortoise.init(
            config={
                "connections": {
                    "qinglan_bot": f"sqlite://{get_path('mcqq.sqlite3')}"
                },
                "apps": {
                    "qinglan_app": {
                        "models": [models],  # 数据库
                        "default_connection": "qinglan_bot"
                    }
                }
            }
        )
        await Tortoise.generate_schemas()
        await cls.update_server_list()

    @classmethod
    async def get_server(cls, **kwargs):
        """获取 Server 信息"""
        return await Server.get(**kwargs).first()

    # 服务器 相关操作
    @classmethod
    async def add_server(cls, **kwargs):
        """添加 Server 信息"""
        return await Server.add(**kwargs)

    @classmethod
    async def delete_server(cls, server_name) -> bool:
        """删除 Server 信息"""
        if await cls.get_server(server_name=server_name):
            # 还存在该 UP 主订阅，不能删除
            return False
        await Server.delete(server_name=server_name)
        return True

    @classmethod
    async def update_server(cls, conf, switch, **kwargs):
        """更新服务器设置"""
        if await Server.update(kwargs, **{conf: switch}):
            await cls.update_server_list()
            return True
        return False

    # 群 相关操作
    @classmethod
    async def get_group(cls, **kwargs):
        """获取群设置"""
        return await Group.get(**kwargs).first()

    @classmethod
    async def add_group(cls, **kwargs):
        """创建群设置"""
        return await Group.add(**kwargs)

    @classmethod
    async def delete_group(cls, type_id) -> bool:
        """删除群设置"""
        if await cls.get_sub(type="group", type_id=type_id):
            # 当前群还有订阅，不能删除
            return False
        await Group.delete(group_id=type_id)
        return True

    @classmethod
    async def update_group(cls, group_id, switch):
        """设置指定群组权限"""
        if not await Group.update({"group_id": group_id}, send_group_name=switch):
            await cls.add_group(group_id=group_id, send_group_name=switch)

    # 频道 相关操作
    @classmethod
    async def get_guild(cls, **kwargs):
        """获取频道设置"""
        return await Guild.get(**kwargs).first()

    @classmethod
    async def add_guild(cls, **kwargs):
        """创建频道设置"""
        return await Guild.add(**kwargs)

    @classmethod
    async def delete_guild(cls, type_id) -> bool:
        """删除子频道设置"""
        if await cls.get_sub(type="guild", type_id=type_id):
            # 当前频道还有订阅，不能删除
            return False
        await Guild.delete(id=type_id)
        return True

    @classmethod
    async def update_guild(cls, guild_id, channel_id, switch):
        """设置指定群组权限"""
        if not await Guild.update({"guild_id": guild_id, "channel_id": channel_id},
                                  send_group_name=switch):
            await cls.add_guild(guild_id=guild_id, channel_id=channel_id, send_group_name=switch)

    @classmethod
    async def get_guild_type_id(cls, guild_id, channel_id) -> Optional[int]:
        """获取频道订阅 ID"""
        guild = await Guild.get(guild_id=guild_id, channel_id=channel_id).first()
        return guild.id if guild else None

    # 列表 相关操作
    @classmethod
    async def get_sub(cls, **kwargs):
        """获取指定位置的 互通列表"""
        return await Sub.get(**kwargs).first()

    @classmethod
    async def add_sub(cls, *, server_name, **kwargs) -> bool:
        """添加互通服务器"""
        if not await Sub.add(server_name=server_name, **kwargs):
            return False
        if kwargs["type"] == "group":
            await cls.add_group(group_id=kwargs["type_id"], send_group_name=False)
        await cls.add_server(
            server_name=server_name,
            rcon_ip="127.0.0.1",
            rcon_port=25575,
            rcon_password="change_password",
            rcon_msg=False,
            rcon_cmd=False
        )
        await cls.update_server_list()
        return True

    @classmethod
    async def set_sub(cls, conf, switch, **kwargs):
        """开关互通设置"""
        return await Sub.update(kwargs, **{conf: switch})

    @classmethod
    async def delete_sub(cls, server_name, type, type_id) -> bool:
        """删除指定互通记录"""
        if await Sub.delete(server_name=server_name, type=type, type_id=type_id):
            await cls.delete_server(server_name=server_name)
            await cls.update_server_list()
            return True
        # 订阅不存在
        return False

    @classmethod
    async def get_subs(cls, **kwargs):
        return await Sub.get(**kwargs)

    @classmethod
    async def get_sub_list(cls, type, type_id) -> List[Sub]:
        """获取指定位置的互通列表"""
        return await cls.get_subs(type=type, type_id=type_id)

    @classmethod
    async def delete_sub_list(cls, type, type_id):
        """删除指定位置的互通列表"""
        async for sub in Sub.get(type=type, type_id=type_id):
            await cls.delete_sub(server_name=sub.server_name, type=sub.type, type_id=sub.type_id)
        await cls.update_server_list()

    @classmethod
    async def update_server_list(cls):
        """更新需要推送的 服务器 主列表"""
        subs = Sub.all()
        servers = Server.all()
        server_list.clear()
        async for server in servers:
            server_list.append(
                {
                    "server_name": server.server_name,
                    "all_group_list": [],
                    "rcon_msg": server.rcon_msg,
                    "rcon_cmd": server.rcon_cmd,
                }
            )

        for per_server in server_list:
            async for sub in subs:
                if per_server["server_name"] == sub.server_name:
                    # 向全群聊列表里装入每个互通记录
                    per_server["all_group_list"].append(
                        {
                            "type": sub.type,
                            "type_id": sub.type_id,
                            "display_server_name": sub.display_server_name
                        }
                    )


get_driver().on_startup(DB.init)
get_driver().on_shutdown(connections.close_all)



from nonebot import on_command
from nonebot.adapters.onebot.v11 import MessageEvent

from .on_msg.data_source import CLIENTS
from ..utils import permission_check, to_me

connected_servers = on_command("已连接服务器列表", rule=to_me(), priority=3)
connected_servers.__doc__ = """已连接服务器列表（管理一）"""

connected_servers.handle()(permission_check)


@connected_servers.handle()
async def _(event: MessageEvent):
    """发送所有已连接至 WebSocket 的服务器"""
    message = "已连接至 WebSocket 的服务器列表\n\n"

    for per_client in CLIENTS:
        message += f'{per_client["server_name"]}\n'
    await connected_servers.finish(message=message)



from nonebot import on_command
from nonebot.adapters.onebot.v11.event import MessageEvent
from nonebot.params import ArgPlainText

from ..rcon import change_rcon_status
from ...database import DB as db
from ...utils import permission_check, handle_server_name, server_name_check, to_me

rcon_cmd_on = on_command("开启rcon命令", rule=to_me(), priority=3)
rcon_cmd_on.__doc__ = """开启rcon命令 Server（管理员）"""

rcon_cmd_on.handle()(permission_check)

rcon_cmd_on.handle()(handle_server_name)

rcon_cmd_on.got("server_name", prompt="请输入要开启 Rcon_CMD 的服务器")(server_name_check)


@rcon_cmd_on.handle()
async def _(event: MessageEvent, server_name: str = ArgPlainText("server_name")):
    """根据 服务器名 开启 Rcon_CMD"""
    if await db.update_server(
            "rcon_cmd",
            True,
            server_name=server_name,
    ):
        await change_rcon_status(server_name=server_name)
        server = await db.get_server(server_name=server_name)
        assert server is not None
        await rcon_cmd_on.finish(f"已开启 [{server.server_name}] 的Rcon_CMD")
    await rcon_cmd_on.finish(f"服务器（{server_name}）未互通，请先互通后再操作")
